;;; auto-correct.el --- Support for auto-correction -*- lexical-binding: t; -*-

;; Copyright (C) 2017 Ian Dunn

;; Author: Ian Dunn <dunni@gnu.org>
;; Maintainer: Ian Dunn <dunni@gnu.org>
;; Keywords: editing
;; Version: 1.0
;; Created: 11 May 2016
;; Modified: 11 May 2016

;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:

;;; Code:

(eval-when-compile (require 'subr-x))

(defgroup auto-correct nil
  "*Auto correction support."
  :prefix "auto-correct-"
  :group 'editing)

(defvar-local auto-correct-predicate nil
  "Predicate to check for auto-correct abbrevs.")

(defun auto-correct-expand-p ()
  (when auto-correct-predicate (funcall auto-correct-predicate)))

(define-abbrev-table 'auto-correct-abbrev-table nil
  ""
  :enable-function #'auto-correct-expand-p)

;;;###autoload
(defun auto-correct-fix-and-add (p)
  "Call `ispell-word', then create an abbrev for it.
With prefix P, create local abbrev. Otherwise it will
be global.

Originally pulled from endless parenthesis."
  (interactive "P")
  (when-let ((word-before (thing-at-point 'word))
             (correction (ispell-word nil 'quiet)))
    (when (and correction (consp correction))
      ;; correction was entered by hand.
      (setq correction (car correction)))
    (if (and (not (or (eq correction 0)  ;; word was corrected from list
                    (eq correction 'quit))) ;; session was exited
             (not (equal word-before correction))) ;; word was corrected
        (let ((bef (downcase word-before))
              (aft (downcase correction)))
          (define-abbrev
            (if p local-abbrev-table auto-correct-abbrev-table)
            bef aft nil :count 1)
          ;; Save the abbrevs.
          (write-abbrev-file)
          (message "\"%s\" now expands to \"%s\"" bef aft))
      (user-error "No correction made at or before point"))))

;;;###autoload
(defun auto-correct-scan-buffer ()
  "Scans current buffer for misspelled words.

When a misspelled word is found, offers to correct and add."
  (interactive)
  (save-excursion
    (goto-char (point-min))
    (while (forward-word)
      (auto-correct-fix-and-add nil))
    (ispell-pdict-save)))

(defun auto-correct-scan-region (start end)
  (interactive "r")
  (save-restriction
    (narrow-to-region start end)
    (auto-correct-scan-buffer)))

;;;###autoload
(defun auto-correct-scan ()
  "Scans the buffer or region."
  (interactive)
  (if (region-active-p)
      (auto-correct-scan-region (region-beginning) (region-end))
    (auto-correct-scan-buffer)))

;;;###autoload
(define-minor-mode auto-correct-mode
  "\\{auto-correct-map}"
  :group 'auto-correct
  :global t
  :init-value nil
  :lighter " Auto-Correct"
  :keymap
  '(([remap (ispell-word)] . auto-correct-fix-and-add)
    ([remap (ispell)]      . auto-correct-scan)))

(add-to-list 'abbrev-minor-mode-table-alist
             `(auto-correct-mode ,auto-correct-abbrev-table)
             'append
             #'equal)

(provide 'auto-correct)

;;; auto-correct.el ends here
